import 'package:example/database_module.dart';
import 'package:example/gateway/sample_gateway.dart';
import 'package:example/model/sample_module.dart';
import 'package:flutter/material.dart';
import 'package:async_injector/async_injector.dart';

import 'gateway/sample_gateway_impl.dart';

Provider provider;

initProvider() async {
  final builder = ProviderBuilder();
  builder.module(DatabaseModule());
  builder.moduleImpl<SampleGateway>(
    (provider) => SampleGatewayImpl(provider.get()),
  );
  provider = await builder.build();
}

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  await initProvider();
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  MyHomePage({Key key, this.title}) : super(key: key);

  final String title;

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  SampleGateway _sampleGateway = provider.get();
  List<SampleModel> items = [];

  @override
  void initState() {
    super.initState();
    _updateItems();
  }

  void _add() async {
    final dateTime = DateTime.now();
    _sampleGateway.set(
      SampleModel(dateTime.millisecondsSinceEpoch, dateTime.toIso8601String()),
    );
    _updateItems();
  }

  void _delete(SampleModel item) async {
    _sampleGateway.delete(item.id);
    _updateItems();
  }

  void _updateItems() async {
    items = await _sampleGateway.getAll();
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.title),
      ),
      body: ListView.builder(
        itemBuilder: (_, i) {
          final item = items[i];
          return InkWell(
            onTap: () => _delete(item),
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Center(
                child: Text(item.message),
              ),
            ),
          );
        },
        itemCount: items.length,
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: _add,
        tooltip: 'Increment',
        child: Icon(Icons.add),
      ),
    );
  }
}
